c
c_______________________________________________________________________
c
c*****boost
c
c            This subroutine produces Gross deuteron wave functions
c         which have been given an arbitrary boost along the z-axis.
c
c         INPUT
c              pp.........momentum in boosted frame in MeV (calculated from
c                                      the on-shell particle)
c              thetap.....polar angle in boosted frame in radians
c              shztaz.....hyperbolic sine of rapidity of boost
c              chztaz.....hyperbolic cosine of rapidity of boost
c              itype=0....particle 2 on shell
c                   =1....particle 1 on shell
c              irel=1.....relativistic boosted wave function
c                  =0.....nonrelativistic wave function
c
c         OUTPUT
c           aboost(lnuc1,lnuc2,ienrgy,ldeu,iwave)..array containing boosted
c                                                  deuteron wave functions.
c
c             lnuc1=0.........positive nucleon helicity for particle 1
c                  =1.........negative nucleon helicity for particle 1
c             lnuc2=0.........positive nucleon helicity for particle 2
c                  =1.........negative nucleon helicity for particle 2
c             ienrgy=0........positive energy projection
c                   =1........negative energy projection
c             ldeu=-1,0,1.....deuteron helicity
c             iwave=0.........wave function
c                   1.........vertex function
c
c              
      subroutine boost(pp,thetap,shztaz,chztaz,itype,irel,aboost)
c
      implicit real*8 (a-h,o-z)
c
      dimension chi(0:1),aboost(0:1,0:1,0:1,-1:1,0:1),d1(-1:1,-1:1),
     &          dhalfp(0:1,0:1),dhalfm(0:1,0:1),a(0:1,0:1,0:1,0:1)
c
      real*8 mn,mt
      common/dpara/mn,mt
c
      parameter(pi=3.141592653589793)
c
c             calculate deuteron rest frame polar angle and momentum
c
      kappa=2*itype-1
c
        snthp=sin(thetap)
        csthp=cos(thetap)
c
      if(irel.eq.1) then                ! relativistic kinematics
c
        shztap=pp/mn
        chztap=sqrt(shztap**2+1.d0)
c
c        tnth=kappa*shztap*snthp/(kappa*chztaz*shztap*csthp-shztaz*
c     &       chztap)
c        theta=atan(tnth)+(1.-sign(1.d0,tnth))*pi/2.
c
        chzta=chztaz*chztap-kappa*shztaz*shztap*csthp
        shzta=sqrt(chzta**2-1.d0)
        csth=(chztaz*shztap*csthp-kappa*shztaz*chztap)/shzta
        theta=acos(polar_range(csth))
        p=mn*shzta
c              calculate the Wigner rotation angles
c
c
        do i=0,1
          kp=1-2*i
c          sgnfac=kp*shztaz
c          tnchi=shztaz*sin(theta)/(shztaz*chzta
c     &          +kp*cos(theta)*chztaz*shzta)
c          tnchi=shztaz*sin(theta)/(kp*shzta*chztaz
c     &          +shztaz*chzta*cos(theta))
c          chi(i)=atan(tnchi)
          chi(i)=get_chi(shzta,chzta,shztaz,chztaz,theta,kp)
c
c          write(6,1) chip
c
c          if(sgnfac.gt.0.0.and.tnchi.lt.0.0) then
c            chi(i)=chi(i)+pi
c          elseif(sgnfac.lt.0.0.and.tnchi.gt.0.0) then
c            chi(i)=chi(i)-pi
c          endif
        end do
c        test=-shztaz*chztap+chztaz*shztap*csthp
c        write(6,1)thetap,theta,chi
  1     format(8(2x,1pe12.4))
c
      else if(irel.eq.0) then            ! nonrelativistic kinematics
c
        puc=mt*shztaz
        p=sqrt(pp**2-kappa*puc*pp*csthp+0.25d0*puc**2)
        csth=(pp*csthp-0.5d0*kappa*puc)/p
        theta=acos(csth)
        chi(0)=0.0
        chi(1)=0.0
c
      end if
c
c               get deuteron wave functions in rest frame
c
      call deuwav(p,irel,a)
c
c                     get spin one rotation matrix
c
      call rotone(theta,d1)
c
c           get spin one half rotation matrices for two Wigner angles
c
      call rothlf(chi(0),dhalfp)
      call rothlf(chi(1),dhalfm)
c
c                construct boosted wave functions
c
      if(itype.eq.0) then
        do ldeu=-1,1
          do ienrgy=0,1
            do lnuc1=0,1
              do lnuc2=0,1
                do iwave=0,1
                  aboost(lnuc1,lnuc2,ienrgy,ldeu,iwave)=0.0
                  do lp1=0,1
                    do lp2=0,1
                      lp1m2=lp2-lp1
                      aboost(lnuc1,lnuc2,ienrgy,ldeu,iwave)=
     &                    aboost(lnuc1,lnuc2,ienrgy,ldeu,iwave)+
     &                    dhalfp(lnuc1,lp1)*d1(ldeu,lp1m2)*
     &                    dhalfm(lp2,lnuc2)*a(lp2,lp1,ienrgy,iwave)
                      end do
                  end do
                end do
              end do
            end do
          end do
        end do
      else if(itype.eq.1) then
        do ldeu=-1,1
          do ienrgy=0,1
            do lnuc1=0,1
              do lnuc2=0,1
                do iwave=0,1
                  aboost(lnuc1,lnuc2,ienrgy,ldeu,iwave)=0.0
                  do lp1=0,1
                    do lp2=0,1
                      lp1m2=lp2-lp1
                      aboost(lnuc1,lnuc2,ienrgy,ldeu,iwave)=
     &                    aboost(lnuc1,lnuc2,ienrgy,ldeu,iwave)+
     &                    dhalfp(lnuc1,lp1)*d1(ldeu,lp1m2)*
     &                    dhalfm(lp2,lnuc2)*a(lp1,lp2,ienrgy,iwave)
                    end do
                  end do
                end do
              end do
            end do
          end do
        end do
      endif
c
c
      return
      end
c
c____________________________________________________________________________
c
      function polar_range(cs)
c
      implicit none
c
      real*8 polar_range,cs
c
      if(cs.gt.1.d0) then
        polar_range=1.d0
      else if(cs.lt.-1.d0) then
        polar_range=-1.d0
      else
        polar_range=cs
      end if
c
      return
      end
c
c____________________________________________________________________________
c
      function get_chi(shzta,chzta,shztaz,chztaz,theta,kp)
c
      implicit none
c
      real*8 get_chi,polar_range,shzta,chzta,shztaz,chztaz,theta,
     &       sp,cp,snchi,cschi,epsilon,cpp,spp,snchip
      integer kp
c
      cp=chzta*chztaz+kp*shztaz*shzta*cos(theta)
      sp=sqrt(cp**2-1.d0)
      snchi=kp*shztaz*sin(theta)/sp
      cschi=(kp*shztaz*chzta*cos(theta)+shzta*chztaz)/sp
c      print *,snchi,cschi
c
      epsilon=1.d-12
      if (abs(snchi).lt.epsilon) then
        cpp=chzta*chztaz+kp*shztaz*shzta*cos(epsilon)
        spp=sqrt(cp**2-1.d0)
        snchip=kp*shztaz*sin(epsilon)/spp
        get_chi=sign(acos(polar_range(cschi)),snchip)
      else
        get_chi=sign(acos(polar_range(cschi)),snchi)
      end if
c
      return
      end
      
